==> notus_format-1.0.0.tar.gz: lib/src/heuristics.dart <==
import 'package:notus_format/notus_format.dart';
import 'package:quill_format/quill_format.dart';

import 'heuristics/delete_rules.dart';
import 'heuristics/format_rules.dart';
import 'heuristics/insert_rules.dart';

/// Registry for insert, format and delete heuristic rules used by
/// [NotusDocument] documents.
class NotusHeuristics {
  /// Default set of heuristic rules.
  ///
  /// Rule order matters.
  static const NotusHeuristics fallback = NotusHeuristics(
    formatRules: [
      FormatLinkAtCaretPositionRule(),
      ResolveLineFormatRule(),
      ResolveInlineFormatRule(),
      // No need in catch-all rule here since the above rules cover all
      // attributes.
    ],
    insertRules: [
      // Embeds
      InsertEmbedsRule(),
      ForceNewlineForInsertsAroundEmbedRule(),
      // Blocks
      AutoExitBlockRule(), // must go first
      PreserveBlockStyleOnInsertRule(),
      // Lines
      PreserveLineStyleOnSplitRule(),
      ResetLineFormatOnNewLineRule(),
      // Inlines
      AutoFormatLinksRule(),
      PreserveInlineStylesRule(),
      // Catch-all
      CatchAllInsertRule(),
    ],
    deleteRules: [
      EnsureEmbedLineRule(),
      PreserveLineStyleOnMergeRule(),
      CatchAllDeleteRule(),
    ],
  );

  const NotusHeuristics({
    this.formatRules,
    this.insertRules,
    this.deleteRules,
  });

  /// List of format rules in this registry.
  final List<FormatRule>? formatRules;

  /// List of insert rules in this registry.
  final List<InsertRule>? insertRules;

  /// List of delete rules in this registry.
  final List<DeleteRule>? deleteRules;

  /// Applies heuristic rules to specified insert operation based on current
  /// state of Notus [document].
  Delta applyInsertRules(NotusDocument document, int index, Object data) {
    final delta = document.toDelta();
    for (var rule in insertRules!) {
      final result = rule.apply(delta, index, data);
      if (result != null) return result..trim();
    }
    throw StateError('Failed to apply insert heuristic rules: none applied.');
  }

  /// Applies heuristic rules to specified format operation based on current
  /// state of Notus [document].
  Delta applyFormatRules(NotusDocument document, int index, int length, NotusAttribute value) {
    final delta = document.toDelta();
    for (var rule in formatRules!) {
      final result = rule.apply(delta, index, length, value);
      if (result != null) return result..trim();
    }
    throw StateError('Failed to apply format heuristic rules: none applied.');
  }

  /// Applies heuristic rules to specified delete operation based on current
  /// state of Notus [document].
  Delta applyDeleteRules(NotusDocument document, int index, int length) {
    final delta = document.toDelta();
    for (var rule in deleteRules!) {
      final result = rule.apply(delta, index, length);
      if (result != null) return result..trim();
    }
    throw StateError('Failed to apply delete heuristic rules: none applied.');
  }
}

==> notus_format-1.0.2.tar.gz: lib/src/heuristics.dart <==
import 'package:notus_format/notus_format.dart';
import 'package:quill_format/quill_format.dart';

import 'heuristics/delete_rules.dart';
import 'heuristics/format_rules.dart';
import 'heuristics/insert_rules.dart';

/// Registry for insert, format and delete heuristic rules used by
/// [NotusDocument] documents.
class NotusHeuristics {
  /// Default set of heuristic rules.
  ///
  /// Rule order matters.
  static const NotusHeuristics fallback = NotusHeuristics(
    formatRules: [
      FormatLinkAtCaretPositionRule(),
      ResolveLineFormatRule(),
      ResolveInlineFormatRule(),
      // No need in catch-all rule here since the above rules cover all
      // attributes.
    ],
    insertRules: [
      // Embeds
      InsertEmbedsRule(),
      ForceNewlineForInsertsAroundEmbedRule(),
      // Blocks
      AutoExitBlockRule(), // must go first
      PreserveBlockStyleOnInsertRule(),
      // Lines
      PreserveLineStyleOnSplitRule(),
      ResetLineFormatOnNewLineRule(),
      // Inlines
      AutoFormatLinksRule(),
      PreserveInlineStylesRule(),
      // Catch-all
      CatchAllInsertRule(),
    ],
    deleteRules: [
      EnsureEmbedLineRule(),
      PreserveLineStyleOnMergeRule(),
      CatchAllDeleteRule(),
    ],
  );

  const NotusHeuristics({
    this.formatRules = const [],
    this.insertRules = const [],
    this.deleteRules = const [],
  });

  /// List of format rules in this registry.
  final List<FormatRule> formatRules;

  /// List of insert rules in this registry.
  final List<InsertRule> insertRules;

  /// List of delete rules in this registry.
  final List<DeleteRule> deleteRules;

  /// Applies heuristic rules to specified insert operation based on current
  /// state of Notus [document].
  Delta applyInsertRules(NotusDocument document, int index, Object data) {
    final delta = document.toDelta();
    for (var rule in insertRules) {
      final result = rule.apply(delta, index, data);
      if (result != null) return result..trim();
    }
    throw StateError('Failed to apply insert heuristic rules: none applied.');
  }

  /// Applies heuristic rules to specified format operation based on current
  /// state of Notus [document].
  Delta applyFormatRules(
      NotusDocument document, int index, int length, NotusAttribute value) {
    final delta = document.toDelta();
    for (var rule in formatRules) {
      final result = rule.apply(delta, index, length, value);
      if (result != null) return result..trim();
    }
    throw StateError('Failed to apply format heuristic rules: none applied.');
  }

  /// Applies heuristic rules to specified delete operation based on current
  /// state of Notus [document].
  Delta applyDeleteRules(NotusDocument document, int index, int length) {
    final delta = document.toDelta();
    for (var rule in deleteRules) {
      final result = rule.apply(delta, index, length);
      if (result != null) return result..trim();
    }
    throw StateError('Failed to apply delete heuristic rules: none applied.');
  }
}
